@@ -47,6 +47,8 @@ TEST_DECLARE   (semaphore_3)
 TEST_DECLARE   (tty)
 #ifdef _WIN32
 TEST_DECLARE   (tty_raw)
+TEST_DECLARE   (tty_empty_write)
+TEST_DECLARE   (tty_large_write)
 #endif
 TEST_DECLARE   (tty_file)
 TEST_DECLARE   (tty_pty)
@@ -404,6 +406,8 @@ TASK_LIST_START
   TEST_ENTRY  (tty)
 #ifdef _WIN32
   TEST_ENTRY  (tty_raw)
+  TEST_ENTRY  (tty_empty_write)
+  TEST_ENTRY  (tty_large_write)
 #endif
   TEST_ENTRY  (tty_file)
   TEST_ENTRY  (tty_pty)
